package com.syntevo.openapi.smartsvn.gui.menu;

import java.util.*;
import javax.swing.*;

/**
 * Utility methods regarding the manipluation of SmartSVN's main menu bar.
 */
public final class MenuUtils {

	// Static =================================================================

	public static boolean insertActionBefore(String insertionPoint, String id, KeyStroke accelerator, String name, String toolTip, boolean availableForFileContextMenu, boolean availableForDirectoryContextMenu, MenuContainer menuContainer) {
		final MenuStructure structure = getMenuStructureForActionId(insertionPoint, menuContainer);
		if (structure == null) {
			return false;
		}

		final int insertionIndex = getIndexOfActionId(insertionPoint, structure);
		structure.addAction(insertionIndex, id, accelerator, name, toolTip, availableForFileContextMenu, availableForDirectoryContextMenu);
		return true;
	}

	public static boolean insertActionAfter(String insertionPoint, String id, KeyStroke accelerator, String name, String toolTip, boolean availableForFileContextMenu, boolean availableForDirectoryContextMenu, MenuContainer menuContainer) {
		final MenuStructure structure = getMenuStructureForActionId(insertionPoint, menuContainer);
		if (structure == null) {
			return false;
		}

		final int insertionIndex = getIndexOfActionId(insertionPoint, structure);
		structure.addAction(insertionIndex + 1, id, accelerator, name, toolTip, availableForFileContextMenu, availableForDirectoryContextMenu);
		return true;
	}

	public static MenuStructure getMenuStructureForActionId(String insertionPoint, MenuContainer menuContainer) {
		for (Iterator it = menuContainer.getItems().iterator(); it.hasNext();) {
			final MenuItem item = (MenuItem)it.next();
			if (item instanceof MenuAction && ((MenuAction)item).getId().equals(insertionPoint)) {
				return (MenuStructure)menuContainer;
			}
			if (item instanceof MenuContainer) {
				final MenuStructure foundStructure = getMenuStructureForActionId(insertionPoint, (MenuContainer)item);
				if (foundStructure != null) {
					return foundStructure;
				}
			}
		}

		return null;
	}

	public static int getIndexOfActionId(String insertionPoint, MenuStructure structure) {
		final List items = structure.getItems();
		for (int index = 0; index < items.size(); index++) {
			final MenuItem item = (MenuItem)items.get(index);
			if (item instanceof MenuAction && ((MenuAction)item).getId().equals(insertionPoint)) {
				return index;
			}
		}

		return -1;
	}

	// Setup ==================================================================

	private MenuUtils() {
	}
}
