package com.syntevo.plugin.jira.transport;

import java.io.*;
import java.net.*;
import java.security.*;
import javax.net.ssl.*;

/**
 * @author syntevo GmbH
 */
class JiraClient {

	// Fields =================================================================

	private final JiraConnection connection;

	// Setup ==================================================================

	public JiraClient(JiraConnection connection) {
		this.connection = connection;
	}

	// Accessing ==============================================================

	public final JiraConnection getConnection() {
		return connection;
	}

	public final URL createConnectableURL(final URL url) throws IOException {
		return new URL(url.getProtocol(), url.getHost(), url.getPort(), url.getFile(), new URLStreamHandler() {
			@Override
			protected URLConnection openConnection(URL u) throws IOException {
				final URLConnection connection = new URL(url.getProtocol(), url.getHost(), url.getPort(), url.getFile()).openConnection();
				configureConnection(connection);
				final PrintWriter log = JiraClient.this.connection.getLog();
				if (log == null) {
					return connection;
				}

				return new JiraDebugLoggingConnection(u, (HttpURLConnection)connection, log);
			}
		});
	}

	// Utils ==================================================================

	private URLConnection configureConnection(URLConnection connection) throws IOException {
		if (!(connection instanceof HttpsURLConnection)) {
			return connection;
		}

		final SSLContext context = createSSLContext(this.connection);
		((HttpsURLConnection)connection).setHostnameVerifier(new JiraHttpsHostnameVerifier());
		((HttpsURLConnection)connection).setSSLSocketFactory(context.getSocketFactory());
		return connection;
	}

	private static SSLContext createSSLContext(JiraConnection connection) throws IOException {
		try {
			final SSLContext context = SSLContext.getInstance("SSL");
			final KeyManager[] keyManagers = new KeyManager[] {new JiraHttpsKeyManager(connection)};
			final TrustManager[] trustManagers = new TrustManager[] {new JiraHttpsTrustManager(connection)};
			context.init(keyManagers, trustManagers, null);
			return context;
		}
		catch (NoSuchAlgorithmException ex) {
			throw new IOException(ex.getMessage());
		}
		catch (KeyManagementException ex) {
			throw new IOException(ex.getMessage());
		}
	}
}